//===- my_case.h ----------------------------------------------------------===//
//
//                              The SkyPat Team
//
// This file is distributed under the New BSD License.
// See LICENSE for details.
//
//===----------------------------------------------------------------------===//
//
// A sample program demonstrating using SkyPat performance testing framework.
//
// Author: Luba Tang <luba@skymizer.com>
//===----------------------------------------------------------------------===//
#ifndef SKYPAT_SAMPLE1_MY_CASE_H
#define SKYPAT_SAMPLE1_MY_CASE_H

// Returns the Nth fibonacci number. For negative n, fibonacci(n) is defined to
// be 1.
int fibonacci(int n);

// Returns n! (the factorial of n). For negative n, n! is defined to be 1.
int factorial(int n);

#endif
